@@ -1,8 +1,17 @@
+/* vim: colorcolumn=80 ts=4 sw=4
+ */
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
 /* converter.c - conversion dialog
 
    Copyright (C) 1998 - 2004 Free Software Foundation
 
+   Copyright © 2005-2020 Various individual contributors, including
+   but not limited to: Jonathon Jongsma, Kalev Lember, who continued
+   to maintain the source code under the licensing terms described
+   herein and below.
+
+   Copyright © 2021 Logan Rathbone <poprocks@gmail.com>
+
    GHex is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
@@ -18,14 +27,13 @@
    If not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-   Authors:
-   Jaka Mocnik <jaka@gnu.org>
+   Original authors of this file:
+   Jaka Mocnik <jaka@gnu.org> (Original GHex author)
    Chema Celorio <chema@gnome.org>
 */
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif /* HAVE_CONFIG_H */
+#include "gtkhex.h"
+#include "converter.h"
 
 #include <ctype.h>      /* for isdigit */
 #include <stdlib.h>     /* for strtoul */
@@ -34,15 +42,44 @@
 #include <gdk/gdkkeysyms.h>
 #include <glib/gi18n.h>
 
-#include "gtkhex.h"
-#include "converter.h"
-#include "ghex-window.h"
+#include <config.h>
+
+/* OPAQUE DATATYPES */
+
+typedef struct _Converter {
+	GtkWidget *window;
+	HexWidget *gh;
+	GtkWidget *entry[5];
+	GtkWidget *close;
+	GtkWidget *get;
+
+	gulong value;
+} Converter;
+
+/* FUNCTION DECLARATIONS */
 
 static void conv_entry_cb(GtkEntry *, gint);
 static void get_cursor_val_cb(GtkButton *button, Converter *conv);
 static void set_values(Converter *conv, gulong val);
+static gchar * clean(gchar *ptr);
 
-Converter *converter = NULL;
+/* STATIC GLOBALS */
+
+static Converter *converter = NULL;
+
+/* --- */
+
+/* reimplement this old function to avoid headaches. FIXME - make this common,
+ * perhaps?
+ */
+static void
+my_gtk_entry_set_text (GtkEntry *entry, char *text)
+{
+	GtkEntryBuffer *buffer;
+
+	buffer = gtk_entry_get_buffer (entry);
+	gtk_entry_buffer_set_text (buffer, text, -1);
+}
 
 static gboolean
 is_char_ok(signed char c, gint base)
@@ -59,7 +96,7 @@ is_char_ok(signed char c, gint base)
 
 	c = c - '0';
 
-	return((c < 0) ||(c >(base - 1))) ? FALSE : TRUE;
+	return ((c < 0) ||(c >(base - 1))) ? FALSE : TRUE;
 }
 
 static void
@@ -73,7 +110,6 @@ entry_filter(GtkEditable *editable, const gchar *text, gint length,
 	/* thou shalt optimize for the common case */
 	if(length == 1) {
 		if(!is_char_ok(*text, base)) {
-			gdk_beep();
 			g_signal_stop_emission_by_name(G_OBJECT(editable), "insert_text");
 		}
 		return;
@@ -91,226 +127,137 @@ entry_filter(GtkEditable *editable, const gchar *text, gint length,
 	if(l == length)
 		return;
 
-	gdk_beep();
 	g_signal_stop_emission_by_name(G_OBJECT(editable), "insert_text");
-#if 0 /* Pasting is not working. gtk is doing weird things. Chema */
-	g_signal_handler_block_by_func (editable, entry_filter, data);
-	g_print("Inserting -->%s<--- of length %i in pos %i\n", s, l, *pos);
-	gtk_editable_insert_text(editable, s, l, pos);
-	g_signal_handler_unblock_by_func (editable, entry_filter, data);
-#endif 
-}
-
-static gint
-entry_key_press_event_cb(GtkWidget *widget, GdkEventKey *event,
-						 gpointer not_used)
-{
-	/* For gtk_entries don't let the gtk handle the event if
-	 * the alt key was pressed. Otherwise the accelerators do not work cause
-	 * gtk takes care of this event
-	 */
-	if(event->state & GDK_MOD1_MASK)
-		g_signal_stop_emission_by_name(G_OBJECT(widget), "key_press_event");
-
-	return FALSE;
 }
 
 #define BUFFER_LEN 40
 
 static GtkWidget *
-create_converter_entry(const gchar *name, GtkWidget *grid,
-						GtkAccelGroup *accel_group, gint pos, gint base)
+create_converter_entry(const gchar *name, GtkWidget *grid, gint pos, gint base)
 {
 	GtkWidget *label;
     GtkWidget *entry;
-#if 0
-    gint accel_key;
-#endif /* 0/1 */
     gchar str[BUFFER_LEN + 1];
 
 	/* label */
 	label = gtk_label_new_with_mnemonic(name);
-	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
 	gtk_grid_attach (GTK_GRID (grid), label, 0, pos, 1, 1);
-#if 0
-	accel_key = gtk_label_parse_uline(GTK_LABEL(label), name);
-#endif /* 0/1 */
-	gtk_widget_show(label);
 
 	/* entry */
 	entry = gtk_entry_new();
 	g_signal_connect(G_OBJECT(entry), "activate",
 					 G_CALLBACK(conv_entry_cb), GINT_TO_POINTER(base));
-	g_signal_connect(G_OBJECT(entry), "key_press_event",
-					 G_CALLBACK(entry_key_press_event_cb), NULL);
-	g_signal_connect(G_OBJECT(entry), "insert_text",
+	g_signal_connect(G_OBJECT(entry), "insert-text",
 					 G_CALLBACK(entry_filter), GINT_TO_POINTER(base));
+
 	gtk_label_set_mnemonic_widget(GTK_LABEL(label), entry);
-#if 0
-	gtk_widget_add_accelerator(entry, "grab_focus", accel_group, accel_key,
-							   GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);
-#endif /* 0/1 */
 	gtk_widget_set_hexpand (entry, TRUE);
 	gtk_grid_attach (GTK_GRID (grid), entry, 1, pos, 1, 1);
-	gtk_widget_show(entry);
-
-	if (GTK_IS_ACCESSIBLE (gtk_widget_get_accessible (entry))) {
-		g_snprintf (str, BUFFER_LEN, "Displays the value at cursor in %s", name+1);
-		add_atk_namedesc (entry, name+1, str);
-		add_atk_relation (entry, label, ATK_RELATION_LABELLED_BY);
-	}
        
 	return entry;
 }
        
-static GtkWidget *
-create_converter_button(const gchar *name, GtkAccelGroup *accel_group)
-{
-	GtkWidget *button;
-    GtkWidget *label;
-#if 0
-    gint accel_key;
-#endif /* 0/1 */
-
-	button = gtk_button_new();
-    label = gtk_label_new_with_mnemonic(name);
-	gtk_misc_set_alignment(GTK_MISC(label), 0.5, 0.5);
-#if 0
-	accel_key = gtk_label_parse_uline(GTK_LABEL(label), name);
-#endif /* 0/1 */
-	gtk_container_add(GTK_CONTAINER(button), label);
-#if 0
-	gtk_widget_add_accelerator(button, "clicked", accel_group, accel_key,
-							   GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);
-#endif /* 0/1 */
-	gtk_label_set_mnemonic_widget(GTK_LABEL(label), button);
-
-	gtk_widget_show(label);
-	gtk_widget_show(button);
-
-	return button;
-}    
-
-/* Made global. We need to access this in ui.c */
-GtkWidget *converter_get = NULL;
-
 static void
-close_converter(GtkWidget *dialog, gint response, gpointer user_data)
+close_converter(GtkWidget *dialog, int response_id, gpointer user_data)
 {
-	ghex_window_sync_converter_item(NULL, FALSE);
-	gtk_widget_hide(GTK_WIDGET(dialog));
+	g_signal_emit_by_name (dialog, "close", G_TYPE_NONE);
 }
 
-static gboolean
-converter_delete_event_cb(GtkWidget *widget, GdkEventAny *e, gpointer user_data)
-{
-	ghex_window_sync_converter_item(NULL, FALSE);
-	gtk_widget_hide(widget);
-	return TRUE;
-}
-
-static gboolean
-conv_key_press_cb (GtkWidget *widget, GdkEventKey *e, gpointer user_data)
-{
-	if (e->keyval == GDK_KEY_Escape) {
-		converter_delete_event_cb(widget, (GdkEventAny *)e, user_data);
-		return TRUE;
-	}
-	return FALSE;
-}
-
-Converter *
-create_converter()
+GtkWidget *create_converter (GtkWindow *parent_win, /* can-NULL */
+		HexWidget *gh)
 {
 	Converter *conv;
 	GtkWidget *grid;
-	GtkAccelGroup *accel_group;
-	gint i;
+	GtkWidget *converter_get;
+	int i;
  
 	conv = g_new0(Converter, 1);
 
+	/* set global for usage in other functions. */
+	converter = conv;
+
+	/* set struct's HexWidget widget */
+	g_assert (HEX_IS_WIDGET(gh));
+	conv->gh = gh;
+
 	conv->window = gtk_dialog_new_with_buttons(_("Base Converter"),
+						/* don't set modal _now_; will be done by app window */
 											   NULL, 0,
-											   GTK_STOCK_CLOSE,
-											   GTK_RESPONSE_OK,
+											   _("_Close"),
+											   GTK_RESPONSE_CLOSE,
 											   NULL);
+
+	if (parent_win) {
+		g_assert (GTK_IS_WINDOW (parent_win));
+
+		gtk_window_set_transient_for (GTK_WINDOW(conv->window), parent_win);
+	}
+
 	g_signal_connect(G_OBJECT(conv->window), "response",
 					 G_CALLBACK(close_converter), conv->window);
 
 	grid = gtk_grid_new ();
+	gtk_widget_set_name (grid, "converter-grid");
 	gtk_grid_set_row_spacing (GTK_GRID (grid), 4);
 	gtk_grid_set_column_spacing (GTK_GRID (grid), 4);
-	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (conv->window))), grid,
-					   TRUE, TRUE, 0);
-	gtk_container_set_border_width(GTK_CONTAINER(gtk_dialog_get_content_area(GTK_DIALOG(conv->window))),
-								   4);
-	gtk_box_set_spacing(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(conv->window))), 2);
-	gtk_widget_show (grid);
+	gtk_box_append (GTK_BOX(gtk_dialog_get_content_area (GTK_DIALOG(conv->window))),
+			grid);
+	gtk_box_set_spacing(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(conv->window))),
+			2);
 	
-	accel_group = gtk_accel_group_new();
-
 	/* entries */
 	conv->entry[0] = create_converter_entry (_("_Binary:"), grid,
-											 accel_group, 0, 2);
+											 0, 2);
 	conv->entry[1] = create_converter_entry (_("_Octal:"), grid,
-											 accel_group, 1, 8);
+											 1, 8);
 	conv->entry[2] = create_converter_entry (_("_Decimal:"), grid,
-											 accel_group, 2, 10);
+											 2, 10);
 	conv->entry[3] = create_converter_entry (_("_Hex:"), grid,
-											 accel_group, 3, 16);
+											 3, 16);
 	conv->entry[4] = create_converter_entry (_("_ASCII:"), grid,
-											 accel_group, 4, 0);
+											 4, 0);
 
 	/* get cursor button */
-	converter_get = create_converter_button(_("_Get cursor value"), accel_group);
+	converter_get = gtk_button_new_with_mnemonic (_("_Get cursor value"));
 
-	g_signal_connect(G_OBJECT(conv->window), "delete_event",
-					 G_CALLBACK(converter_delete_event_cb), conv);
 	g_signal_connect(G_OBJECT(converter_get), "clicked",
 					 G_CALLBACK(get_cursor_val_cb), conv);
-	g_signal_connect(G_OBJECT(conv->window), "key_press_event",
-					 G_CALLBACK(conv_key_press_cb), conv);
+
 	gtk_grid_attach (GTK_GRID (grid), converter_get, 0, 5, 2, 1);
 
-	/* add the accelerators */
-	gtk_window_add_accel_group(GTK_WINDOW(conv->window), accel_group);
+	gtk_accessible_update_property (GTK_ACCESSIBLE(converter_get),
+			GTK_ACCESSIBLE_PROPERTY_DESCRIPTION,
+			_("Gets the value at cursor in binary, octal, decimal, hex and ASCII"),
+			-1);
 
-	if (GTK_IS_ACCESSIBLE(gtk_widget_get_accessible(converter_get))) {
-		add_atk_namedesc (converter_get, _("Get cursor value"), _("Gets the value at cursor in binary, octal, decimal, hex and ASCII"));
-		for (i=0; i<5; i++) {
-			add_atk_relation (conv->entry[i], converter_get, ATK_RELATION_CONTROLLED_BY);
-			add_atk_relation (converter_get, conv->entry[i], ATK_RELATION_CONTROLLER_FOR);
-		}
-	}
-
-	return conv;
+	return conv->window;
 }
 
 static void
 get_cursor_val_cb(GtkButton *button, Converter *conv)
 {
-	GtkWidget *view;
 	guint val, start;
-	GHexWindow *win = ghex_window_get_active();
+	guint group_type;
+	HexDocument *doc;
+	size_t payload;
 
-	if(win == NULL || win->gh == NULL)
-		return;
+	g_return_if_fail (HEX_IS_WIDGET(conv->gh));
 
-	view = GTK_WIDGET(win->gh);
+	doc = hex_widget_get_document (conv->gh);
+	payload = hex_buffer_get_payload_size (hex_document_get_buffer (doc));
+	group_type = hex_widget_get_group_type (conv->gh);
+	start = hex_widget_get_cursor (conv->gh);
+	start = start - start % group_type;
 
-	if(view) {
-		start = gtk_hex_get_cursor(GTK_HEX(view));
-		start = start - start % GTK_HEX(view)->group_type;
-		val = 0;
-		do {
-			val <<= 8;
-			val |= gtk_hex_get_byte(GTK_HEX(view), start);
-			start++;
-		} while((start % GTK_HEX(view)->group_type != 0) &&
-				(start < GTK_HEX(view)->document->file_size) );
+	val = 0;
+	do {
+		val <<= 8;
+		val |= hex_widget_get_byte(conv->gh, start);
+		start++;
+	} while((start % group_type != 0) &&
+			(start < payload) );
 
-		set_values(conv, val);
-	}
+	set_values(conv, val);
 }
 
 static gchar *
@@ -344,13 +291,13 @@ set_values(Converter *conv, gulong val)
 	for(i = 0; i < 32; i++)
 		buffer[i] =((val & (1L << (31 - i)))?'1':'0');
 	buffer[i] = 0;
-	gtk_entry_set_text(GTK_ENTRY(conv->entry[0]), clean(buffer));
+	my_gtk_entry_set_text(GTK_ENTRY(conv->entry[0]), clean(buffer));
 
 	g_snprintf(buffer, CONV_BUFFER_LEN, "%o",(unsigned int)val);
-	gtk_entry_set_text(GTK_ENTRY(conv->entry[1]), buffer);
+	my_gtk_entry_set_text(GTK_ENTRY(conv->entry[1]), buffer);
 	
 	g_snprintf(buffer, CONV_BUFFER_LEN, "%lu", val);
-	gtk_entry_set_text(GTK_ENTRY(conv->entry[2]), buffer);
+	my_gtk_entry_set_text(GTK_ENTRY(conv->entry[2]), buffer);
 
 	for(i = 0, tmp = val; i < nhex; i++) {
 		buffer[nhex - i - 1] = (tmp & 0x0000000FL);
@@ -361,7 +308,7 @@ set_values(Converter *conv, gulong val)
 		tmp = tmp >> 4;
 	}
 	buffer[i] = '\0';
-	gtk_entry_set_text(GTK_ENTRY(conv->entry[3]), buffer);
+	my_gtk_entry_set_text(GTK_ENTRY(conv->entry[3]), buffer);
 	
 	for(i = 0, tmp = val; i < nbytes; i++) {
 		buffer[nbytes - i - 1] = tmp & 0x000000FF;
@@ -370,7 +317,7 @@ set_values(Converter *conv, gulong val)
 		tmp = tmp >> 8;
 	}
 	buffer[i] = 0;
-	gtk_entry_set_text(GTK_ENTRY(conv->entry[4]), buffer);
+	my_gtk_entry_set_text(GTK_ENTRY(conv->entry[4]), buffer);
 }
 
 static void
@@ -379,12 +326,16 @@ conv_entry_cb(GtkEntry *entry, gint base)
 	gchar buffer[33];
 	const gchar *text;
 	gchar *endptr;
-	gulong val = converter->value;
+	gulong val;
 	int i, len;
 	
-	text = gtk_entry_get_text(entry);
+	g_return_if_fail (converter);
+
+	/* shorthand. */
+	val = converter->value;
+	text = gtk_entry_buffer_get_text (gtk_entry_get_buffer (entry));
 	
-	switch(base) {
+	switch (base) {
 	case 0:
 		strncpy(buffer, text, 4);
 		buffer[4] = 0;
@@ -416,7 +367,8 @@ conv_entry_cb(GtkEntry *entry, gint base)
 		if(*endptr != 0) {
 			converter->value = 0;
 			for(i = 0; i < 5; i++)
-				gtk_entry_set_text(GTK_ENTRY(converter->entry[i]), _("ERROR"));
+				my_gtk_entry_set_text(GTK_ENTRY(converter->entry[i]),
+						_("ERROR"));
 			gtk_editable_select_region(GTK_EDITABLE(entry), 0, -1);
 			return;
 		}
