@@ -168,6 +168,8 @@ struct _HexWidget
 	PangoLayout *xlayout, *alayout, *olayout;
 
 	GtkWidget *context_menu;
+	GtkWidget *geometry_popover;
+	GtkWidget *auto_geometry_checkbtn, *cpl_spinbtn;
 
 	GtkAdjustment *adj;
 
@@ -524,6 +526,25 @@ toggle_ascii_action (GtkWidget *widget,
 	}
 }
 
+static void
+update_geometry (HexWidget *self, gpointer data)
+{
+	if (gtk_check_button_get_active (GTK_CHECK_BUTTON(self->auto_geometry_checkbtn)))
+		hex_widget_set_geometry (self, 0, 0);
+	else
+		hex_widget_set_geometry (self, gtk_spin_button_get_value (GTK_SPIN_BUTTON(self->cpl_spinbtn)), 0);
+}
+
+static void
+geometry_action (GtkWidget *widget,
+		const char *action_name,
+		GVariant *parameter)
+{
+	HexWidget *self = HEX_WIDGET (widget);
+
+	gtk_popover_popup (GTK_POPOVER(self->geometry_popover));
+}
+
 /*
  * ?_to_pointer translates mouse coordinates in hex/ascii view
  * to cursor coordinates.
@@ -1814,8 +1835,11 @@ key_press_cb (GtkEventControllerKey *controller,
 	{
 		case GDK_KEY_BackSpace:
 			if (self->cursor_pos > 0) {
-				hex_document_set_data (self->document, self->cursor_pos - 1,
-						0, 1, NULL, TRUE);
+				if (self->insert)
+					hex_widget_delete_selection (self);
+				else
+					hex_widget_zero_selection (self);
+
 				if (self->selecting)
 					self->selecting = FALSE;
 				hex_widget_set_cursor (self, self->cursor_pos - 1);
@@ -1824,9 +1848,13 @@ key_press_cb (GtkEventControllerKey *controller,
 			break;
 
 		case GDK_KEY_Delete:
-			if (self->cursor_pos < payload_size) {
-				hex_document_set_data (self->document, self->cursor_pos,
-						0, 1, NULL, TRUE);
+			if (self->cursor_pos < payload_size)
+			{
+				if (self->insert)
+					hex_widget_delete_selection (self);
+				else
+					hex_widget_zero_selection (self);
+
 				hex_widget_set_cursor (self, self->cursor_pos);
 				ret = GDK_EVENT_STOP;
 			}
@@ -2391,7 +2419,10 @@ hex_widget_real_cut_to_clipboard(HexWidget *self,
 {
 	if (self->selection.start != -1 && self->selection.end != -1) {
 		hex_widget_real_copy_to_clipboard(self);
-		hex_widget_delete_selection(self);
+		if (self->insert)
+			hex_widget_delete_selection(self);
+		else
+			hex_widget_zero_selection(self);
 	}
 }
 
@@ -2565,6 +2596,7 @@ hex_widget_dispose (GObject *object)
 	g_clear_pointer (&self->busy_spinner, gtk_widget_unparent);
 
 	g_clear_pointer (&self->context_menu, gtk_widget_unparent);
+	g_clear_pointer (&self->geometry_popover, gtk_widget_unparent);
 
 	/* Clear pango layouts
 	 */
@@ -2752,6 +2784,10 @@ hex_widget_class_init (HexWidgetClass *klass)
 			NULL,
 			toggle_ascii_action);
 
+	gtk_widget_class_install_action (widget_class, "gtkhex.geometry",
+			NULL,
+			geometry_action);
+
 	/* SHORTCUTS FOR ACTIONS (not to be confused with keybindings, which are
 	 * set up in hex_widget_init) */
 
@@ -2948,12 +2984,22 @@ hex_widget_init (HexWidget *self)
 
 	/* Context Menu */
 
-	builder = gtk_builder_new_from_resource (RESOURCE_BASE_PATH "/context-menu.ui");
+	builder = gtk_builder_new_from_resource (RESOURCE_BASE_PATH "/gtkhex.ui");
 	menu = G_MENU_MODEL(gtk_builder_get_object (builder, "context-menu"));
 	self->context_menu = gtk_popover_menu_new_from_model (menu);
 
 	gtk_widget_set_parent (self->context_menu, widget);
 
+	self->geometry_popover = GTK_WIDGET(gtk_builder_get_object (builder, "geometry_popover"));
+
+	self->auto_geometry_checkbtn = GTK_WIDGET(gtk_builder_get_object (builder, "auto_geometry_checkbtn"));
+	self->cpl_spinbtn = GTK_WIDGET(gtk_builder_get_object (builder, "cpl_spinbtn"));
+
+	g_signal_connect_swapped (self->auto_geometry_checkbtn, "toggled", G_CALLBACK(update_geometry), self);
+	g_signal_connect_swapped (self->cpl_spinbtn, "value-changed", G_CALLBACK(update_geometry), self);
+
+	gtk_widget_set_parent (self->geometry_popover, widget);
+
 	g_object_unref (builder);
 
 	/* Initialize Adjustment */
@@ -3269,6 +3315,41 @@ hex_widget_delete_selection (HexWidget *self)
 	hex_widget_set_cursor (self, start);
 }
 
+static char *zeros = NULL;
+
+/**
+ * hex_widget_zero_selection:
+ *
+ * Set the current selection to zero. The resulting action will be undoable.
+ *
+ * Since: 4.4
+ */
+void
+hex_widget_zero_selection (HexWidget *self)
+{
+	gint64 start, end;
+	size_t len;
+	size_t written = 0;
+
+	start = MIN(self->selection.start, self->selection.end);
+	end = MAX(self->selection.start, self->selection.end);
+
+	len = end - start + 1;
+	g_return_if_fail (len);
+
+	if (zeros == NULL) {
+		zeros = g_malloc0 (512);
+	}
+
+	while (written < len) {
+		size_t batch_size = len < 512 ? len : 512;
+		hex_document_set_data (self->document,
+				start, batch_size, batch_size,
+				zeros, TRUE);
+		written += batch_size;
+	}
+}
+
 /**
  * hex_widget_set_nibble:
  * @lower_nibble: %TRUE if the lower nibble of the current byte should be
@@ -3564,7 +3645,7 @@ hex_widget_set_insert_mode (HexWidget *self, gboolean insert)
 	self->insert = insert;
 
 	if (!self->insert && self->cursor_pos > 0 && self->cursor_pos >= payload_size)
-			self->cursor_pos = payload_size - 1;
+			hex_widget_set_cursor (self, payload_size - 1);
 }
 
 /**
